(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2005-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: communication.ml,v 1.15 2017/05/31 15:54:22 fclement Exp $ *)

type values = Typed_value.t array;;

type service_name =
  | Pio_Ok
  | Pio_Ko
  | Pio_Allo
  | Pio_Bye
  | Pio_Start
  | Pio_Stop
;;

type task_name = string;;

type t =
  | Pio_Phrase of values
  | Pio_Task of (task_name * values)
  | Pio_Result of values
  | Pio_Error of values
  | Pio_Service of (service_name * values)
and communication = t
;;

exception Wrong_communication of Upervasives.message;;

let service_name_of_string = function
  | "Ok" -> Pio_Ok
  | "Ko" -> Pio_Ko
  | "Allo" -> Pio_Allo
  | "Bye" -> Pio_Bye
  | "Start" -> Pio_Start
  | "Stop" -> Pio_Stop
  | sname ->
      raise
        (Wrong_communication (Printf.sprintf "Unknown service kind %s" sname))
;;

let string_of_service_name = function
  | Pio_Ok -> "Ok"
  | Pio_Ko -> "Ko"
  | Pio_Allo -> "Allo"
  | Pio_Bye -> "Bye"
  | Pio_Start -> "Start"
  | Pio_Stop -> "Stop"
;;
