(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2005-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: read_communication.mli,v 1.14 2017/05/31 15:54:22 fclement Exp $ *)

(** Reading communication. *)

(** We read [Communication.t] values according to the global format mode
 flag. *)

exception Pio_closed_input_channel;;

val read_communication : Scanf.Scanning.scanbuf -> Communication.t
(** General function to read a communication from a [Scanf.Scanning] input
 channel.
 In case of wrong communication, the function returns a service communication
 reporting the exception raised while reading the wrong communication.
 Raise [Pio_closed_input_channel] when the [Scanf.Scanning] input channel is
 closed before reading any character.
*)
;;

(**/**)

val unsafe_read_communication : Scanf.Scanning.scanbuf -> Communication.t
(** Unsafe function to read a communication from a [Scanf.Scanning] input
 channel.
 This function may raise exceptions [Wrong_communication],
 [Scanf.Scan_failure], [Failure] and [End_of_file]. *)
;;
