(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2005-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(*  $Id: main.ml,v 1.13 2017/05/31 15:54:23 fclement Exp $ *)

module Ok = Uoption_key;;
module Co = Ucommand_option;;

open Write_typed_value;;
open Write_communication;;

type output_mode =
   | Output_communication
   | Output_typed_value
;;

let { Uaref.get = get_output_mode; set = set_output_mode; } =
  Uaref.make_aref_single "output_mode" Output_communication
;;

let set_communication () = set_output_mode Output_communication;;
let set_typed_value () = set_output_mode Output_typed_value;;

let communication_rco =
  {
   Co.key = Ok.of_string "--communication";
   spec = Arg.Unit set_communication;
   doc =
     "  Turn on the generation of a communication r/w test file.\
     \n\tThis is the default.\n";
 }

and typed_value_rco =
  {
   Co.key = Ok.of_string "--typed-value";
   spec = Arg.Unit set_typed_value;
   doc = "  Turn on the generation of a typed_value r/w test file.\n";
  }
;;

let add_options () =
  Co.application_add_all_rco
    [
     communication_rco;
     typed_value_rco;
    ]
;;

add_options ();;

(* Check general write communications on a buffer *)

let write_and_flush write ob t =
  write ob t;
  Ubuffer.flush_buffer_on_stdout ob;
;;

let main () =
  Co.parse ();
  let ob = Buffer.create 1024 in
  begin 
    if get_output_mode () = Output_typed_value
    then
      Ulist.iter
        (write_and_flush write_typed_value ob)
        Gen_typed_values.list
    else
      Ulist.iter
        (write_and_flush write_communication ob)
        Gen_communication.list
  end;
  exit 0
;;

main ();;
