(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2005-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: value_type.mli,v 1.12 2017/05/31 15:54:22 fclement Exp $ *)

(** {3 The types of typed values used in communications} *)

open Value_type_def;;

(** {6 Some predefined interesting value types} *)

(** These are convenient shortcuts for usual types. *)

val bool_couple_type : scalar_type;;
val bool_triple_type : scalar_type;;
val int_couple_type : scalar_type;;
val int_triple_type : scalar_type;;
val native_int_couple_type : scalar_type;;
val native_int_triple_type : scalar_type;;
val int32_couple_type : scalar_type;;
val int32_triple_type : scalar_type;;
val int64_couple_type : scalar_type;;
val int64_triple_type : scalar_type;;
val float_couple_type : scalar_type;;
val float_couple_bin_type : scalar_type;;
val float_triple_type : scalar_type;;
val float_triple_bin_type : scalar_type;;

(** {6 Reader/printer for value types} *)

val to_ob : Buffer.t -> value_type -> unit
(** [to_ob ob vtyp] prints value type [vtyp] into output buffer [ob]. *)
;;
val of_ib : Scanf.Scanning.scanbuf -> value_type
(** [of_ib ib] reads a value type from scanning buffer [ib]. *)
;;
