#######################################################################
#                                                                     #
#                                 Pio                                 #
#                                                                     #
#                    Polyglot Input/Output Library                    #
#                                                                     #
#               EPI Pomdapi, INRIA Paris - Rocquencourt               #
#                                                                     #
#  Copyright (c) 2005-2015 INRIA.                                     #
#  Institut National de Recherche en Informatique et en Automatique.  #
#  All rights reserved. This file is distributed only by permission.  #
#                                                                     #
#  Francois Clement <Francois.Clement@inria.fr>                       #
#  Pierre Weis <Pierre.Weis@inria.fr>                                 #
#                                                                     #
#######################################################################

# $Id: Pio_configure_additional_options.profile,v 1.4 2016/12/12 19:39:19 fclement Exp $

PIO_AVAILABLE_IMPLEMENTATIONS="c c++ fortran ocaml"

pio_process_additional_options () {
    UCSB="${USER_CONFIGURE_SETTINGS_BNAME:?}"
    report_message "Setting Pio bindings in $UCSB"

    # Compute the list of supported implementations.
    PIO_SUPPORTED_IMPLEMENTATIONS=""
    PIO_UNSUPPORTED_IMPLEMENTATIONS=""
    for IMPL in $PIO_AVAILABLE_IMPLEMENTATIONS; do
        case $IMPL in
        ( c ) COMPILERS="$C_COMPILER";;
        ( c++ ) COMPILERS="$CPP_COMPILER";;
        ( fortran ) COMPILERS="$FORTRAN_COMPILER";;
        ( ocaml ) COMPILERS="$OCAML_BYT_COMPILER $OCAML_BIN_COMPILER";;
        ( * )
            internal_error "unknown implementation \"$IMPL\""
            exit $INTERNAL_ERROR
            ;;
        esac
        for COMP in $COMPILERS; do
            case "$COMP" in
            ( Not_found | False ) SUPPORTED=false;;
            ( * ) SUPPORTED=true; break;;
            esac
        done
        if $SUPPORTED; then
            if test -z "$PIO_SUPPORTED_IMPLEMENTATIONS"; then
                PIO_SUPPORTED_IMPLEMENTATIONS="$IMPL"
            else
                PSI="$PIO_SUPPORTED_IMPLEMENTATIONS"
                PIO_SUPPORTED_IMPLEMENTATIONS="$PSI $IMPL"
            fi
        else
            if test -z "$PIO_UNSUPPORTED_IMPLEMENTATIONS"; then
                PIO_UNSUPPORTED_IMPLEMENTATIONS="$IMPL"
            else
                PUI="$PIO_UNSUPPORTED_IMPLEMENTATIONS"
                PIO_UNSUPPORTED_IMPLEMENTATIONS="$PUI $IMPL"
            fi
        fi
    done
    case "$PIO_SUPPORTED_IMPLEMENTATIONS" in
    ( "$PIO_AVAILABLE_IMPLEMENTATIONS" )
        PIO_SUPPORTED_IMPLEMENTATIONS="All";;
    ( "" ) PIO_SUPPORTED_IMPLEMENTATIONS="None";;
    esac
    case "$PIO_UNSUPPORTED_IMPLEMENTATIONS" in
    ( "" ) PIO_UNSUPPORTED_IMPLEMENTATIONS="None";;
    ( "$PIO_AVAILABLE_IMPLEMENTATIONS" )
        PIO_UNSUPPORTED_IMPLEMENTATIONS="All";;
    esac

    # Store the current settings for supported implementations.
    add_to_config_file \
        "${USER_CONFIGURE_SETTINGS_NO_EXT:?}" \
        "Pio available implementations" \
          "PIO_AVAILABLE_IMPLEMENTATIONS" "$PIO_AVAILABLE_IMPLEMENTATIONS" \
        "Pio supported implementations" \
          "PIO_SUPPORTED_IMPLEMENTATIONS" "$PIO_SUPPORTED_IMPLEMENTATIONS" \
        "Pio unsupported implementations" \
          "PIO_UNSUPPORTED_IMPLEMENTATIONS" "$PIO_UNSUPPORTED_IMPLEMENTATIONS"
    IER=$?
    if test $IER -ne 0; then exit $IER; fi
}
