(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(*  This example is based on an example designed for OCamlP3l.         *)
(*                                                                     *)
(***********************************************************************)

(* $Id: options.ml,v 1.10 2017/01/05 16:25:32 fclement Exp $ *)

module Nat = Unatural;;
module A = Uarray;;
module Ar = Uaref;;
module Ok = Uoption_key;;
module Co = Ucommand_option;;

let _0_ = Nat._0_;;

open Rdec2;;
open Dd_config;;
open Gnuplot;;

let configuration_directory_option_name = "configuration directory"
and configuration_filename_option_name = "configuration filename"
;;

let default_configuration_directory = Filename.concat "." "data"
and default_configuration_filename = "config"
;;

let { Ar.get = get_config_dir; set = set_config_dir; } =
  Ar.make_aref_single
    configuration_directory_option_name
    default_configuration_directory

and { Ar.get = get_config_filename; set = set_config_filename; } =
  Ar.make_aref_single
    configuration_filename_option_name
    default_configuration_filename
;;

let get_global_parameters () =
  let cfg_fname =
    Filename.concat (get_config_dir ()) (get_config_filename ()) in
  let cfg = parse_configuration_file cfg_fname in
  let rdec_fname =
    Filename.concat (get_config_dir ()) cfg.rdec_config_filename in
  let rdec = parse_rdec_config_file rdec_fname in
  check_regular_decomposition rdec;
  check_graphical_output cfg.graphical_output;
  (cfg_fname, cfg, rdec)
;;

let configuration_directory_lok =
  Ok.long_option_key_of_string configuration_directory_option_name

and configuration_filename_lok =
  Ok.long_option_key_of_string configuration_filename_option_name
;;

let configuration_directory_rco =
  {
   Co.key = configuration_directory_lok;
   spec = Arg.String set_config_dir;
   doc =
     Printf.sprintf
       "<dir>  Set the %s to <dir>.\
       \n\tDefault directory is %S."
       configuration_directory_option_name
       (get_config_dir ());
  }

and configuration_filename_rco =
  {
   Co.key = configuration_filename_lok;
   spec = Arg.String set_config_filename;
   doc =
     Printf.sprintf
       "<fname>  Set the %s to <fname>.\
       \n\tDefault file is %S."
       configuration_filename_option_name
       (get_config_filename ());
  }
;;

let add_options () =
  Co.application_add_all_rco_auto
    [
     configuration_directory_rco;
     configuration_filename_rco;
    ];
;;
add_options ();;

(* Sys.argv is never empty.
   Therefore, the next A.unsafe_get will be safe. *)
let cmd_name = A.unsafe_get Sys.argv _0_;;
Co.set_usage_msg
  (Printf.sprintf
     "\n%s -- Domain decomposition, solves Poisson's equation"
     cmd_name)
;;
