(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: hello.ml,v 1.23 2017/05/15 13:58:10 fclement Exp $ *)

module Nat = Unatural;;
module Perv = Upervasives;;
module S = Ustring;;

let _10_ = Nat.unsafe_of_int 10;;

let stream_of_string s = Sklstream.of_list (S.scatter s);;

let put_char c = Printf.printf "%c%!" c; c;;

(* The atomic put char skeleton. *)
(*let put_char_skl = skl b -> if b then put_char else Perv.id;;*)
let put_char_skl = Skl.make_skl (fun b -> if b then put_char else Perv.id);;

(* The full computational skeleton expression. *)
let hello_skl b nw = Skl.farm (put_char_skl b, nw);;

let main { Parfun.parfun = parfun; } =
  let is = stream_of_string "Hello world!\n" in
  let os = parfun (hello_skl true _10_) is in
  Sklstream.iter ignore os;
  let os = parfun (hello_skl false _10_) is in
  Sklstream.iter (fun x -> ignore (put_char x)) os;
;;

Parfun.pardo main;;
