(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: helper.mli,v 1.10 2017/01/04 11:58:31 fclement Exp $ *)

val scalar_prod : int array -> int array -> int;;

module IO : sig
  val input_vector : in_channel -> int array option
  (** Input an [int] vector on a given [in_channel]. *)
  ;;

  val input_matrix : in_channel -> int array array option
  (** Input an [int] matrix on a given [in_channel]. *)
  ;;

  val input_fvector : in_channel -> float array option
  (** Input a [float] vector on a given [in_channel]. *)
  ;;

  val input_tridiag : in_channel -> Tridiag.t option
  (** Input a tridiagonal matrix on a given [in_channel]. *)
  ;;

  val output_vector : out_channel -> int array -> unit
  (** Output an [int] vector on a given [out_channel]. *)
  ;;

  val output_raw : out_channel -> float array array -> unit
  (** Output a [float] matrix row by row on a given [out_channel]. *)
  ;;
end
;;

val fetch_std_data : string -> string -> int array array * int array;;
val fetch_tri_data : string -> string -> Tridiag.t * float array;;
