(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: parser.ml,v 1.18 2017/05/23 16:36:28 fclement Exp $ *)

(* Monadic parsers *)
(* Copyright (c) Quentin Carbonneaux 2011 *)

module L = Ulist;;

type ('a, 'b) t = { parse : 'a -> ('b * 'a) option; }
and ('a, 'b) parser = ('a, 'b) t
;;

let parse p s =
  match p.parse s with
  | None -> None
  | Some (x, _) -> Some x
;;

let full_parse p = p.parse;;

let return x = { parse = fun s -> Some (x, s) };;
let zero = { parse = fun _ -> None };;
let bind p f =
  let parse s =
    match p.parse s with
    | None -> None
    | Some (x, s') -> (f x).parse s' in
  { parse; }
;;

let ( >>= ) = bind;;
let ( >> ) p0 p1 = p0 >>= (fun _ -> p1);;

let plus p0 p1 =
  let parse s =
    let ret = p0.parse s in
    if ret <> None then ret else
    p1.parse s in
  { parse; }
;;

let ( +++ ) = plus;;

let liftM f pa = pa >>= fun x -> return (f x);;

let rec any = function
  | [] -> zero
  | p :: ps -> p +++ (any ps)
;;

let many p =
  let rec loop acc =
    (p >>= (fun c -> loop (c :: acc))) +++
    (return (L.rev acc)) in
  loop []
;;

let many1 p = p >>= (fun x -> many p >>= (fun l -> return (x :: l)));;

let many_till pcond p =
  let rec loop acc =
    (pcond >> return (L.rev acc)) +++
    (p >>= fun x -> loop (x :: acc)) in
  loop []
;;

let maybe p = (p >>= (fun x -> return (Some x))) +++ (return None);;

let times n p =
  let rec loop n acc =
    if n = 0 then return (L.rev acc) else
    p >>= (fun x -> loop (pred n) (x :: acc)) in
  loop n []
;;

let sep_by1 psep p =
  let rec loop acc =
    ((psep >> p) >>= (fun x -> loop (x :: acc))) +++
    (return (L.rev acc)) in
  (liftM (fun x -> [ x; ]) p) >>= loop
;;

let less_than n p =
  let rec loop n acc =
    if n = 0 then return (L.rev acc) else
    (p >>= (fun x -> loop (pred n) (x :: acc))) +++
    return (L.rev acc) in
  loop n []
;;
