(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: options.ml,v 1.27 2017/01/04 11:58:31 fclement Exp $ *)

module Nat = Unatural;;
module A = Uarray;;
module Ar = Uaref;;
module Ok = Uoption_key;;
module Co = Ucommand_option;;

let _0_ = Nat._0_;;

let command_option_name = "command"
and command_arguments_option_name = "command arguments"
;;

let default_command = "echo"
and default_command_arguments = "-"
;;

let { Ar.get = get_command; set = set_command; } =
  Ar.make_aref_single
    command_option_name
    default_command

and { Ar.get = get_command_arguments; set = set_command_arguments; } =
  Ar.make_aref_single
    command_arguments_option_name
    default_command_arguments
;;

let command_lok =
  Ok.long_option_key_of_string command_option_name
and command_arguments_lok =
  Ok.long_option_key_of_string command_arguments_option_name
;;

let command_rco =
  {
   Co.key = command_lok;
   spec = Arg.String set_command;
   doc =
     Printf.sprintf
       "<cmd>\
       \n\tSet the command to run. \
       Default is %S."
       (get_command ());
  }

and command_arguments_rco =
  {
   Co.key = command_arguments_lok;
   spec = Arg.String set_command_arguments;
   doc =
     Printf.sprintf
       "<file>\
       \n\tSet the file containing arguments of the command \
       to run in parallel.\
       \n\tUse '-' (dash) to specify the standard input. \
       Default is %S."
       (get_command_arguments ());
  }
;;

let add_options () =
  Co.application_add_all_rco_auto
    [
     command_rco;
     command_arguments_rco;
    ];
;;
add_options ();;

(* Sys.argv is never empty.
   Therefore, the next A.unsafe_get will be safe. *)
let cmd_name = A.unsafe_get Sys.argv _0_;;
Co.set_usage_msg
  (Format.sprintf
     "@[<v>\
      @ \
      %s -- Parallel execution of a command@ \
      @ \
      Usage: %s [<option>...]@ \
      @ \
      Options are:\
      @]@."
     cmd_name
     cmd_name)
;;
