(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: algo.ml,v 1.10 2017/01/04 11:58:31 fclement Exp $ *)

module Nat = Unatural;;

module Bb = Black_box;;

let printer_Int_List = Og_printer.printer_List Og_printer.printer_Int;;

let print_ident_value ident printer ppf value =
  Format.fprintf ppf "%s =@ %a"
    ident
    (printer Og_precedence.Ck_toplevel) value
;;

let report ident printer ppf value =
  Format.fprintf ppf "@[<hv 2>%a@]@."
    (print_ident_value ident printer) value
;;

let report_int_list ident = report ident printer_Int_List;;

let ppf = Format.std_formatter;;

let report_state (i, li) =
  let sli = Printf.sprintf "l%d" (Nat.to_int i) in
  report_int_list sli ppf li;
;;

let init () =
  let l0 = Bb.init () in
  (Nat._0_, l0)
;;

let imax = Nat._5_;;

let stop (i, _) = i >= imax;;

let iteration heavy_comps (i, li) =
  (Nat.succ i, Bb.light_comp 0 :: heavy_comps li)
;;

let rec iterations heavy_comps st_i =
  report_state st_i;
  if stop st_i then st_i else
  let st_ip1 = iteration heavy_comps st_i in
  iterations heavy_comps st_ip1
;;

let final st_n =
  report_state st_n;
  Bb.final ();
;;

let algo heavy_comps =
  let st_0 = init () in
  let st_n = iterations heavy_comps st_0 in
  final st_n;
;;
