(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: skl.mli,v 1.40 2017/05/15 13:58:09 fclement Exp $ *)

(** {3 Skeletons operations} *)

open Tools_common_types;;

type ('a, 'b) sum = ('a, 'b) Sk.sum =
  | Inl of 'a
  | Inr of 'b
;;

type ('a, 'b) t = ('a, 'b) Sk.Skl.t
and ('a, 'b) skel = ('a, 'b) t
(** The type of skeletons. *)
;;

(** {6 Creating skeletons} *)

val id : unit -> ('a, 'a) t
(** Identity is builtin, it is optimized. *)
;;

val make_skl : ('x -> 'a -> 'b) -> 'x -> ('a, 'b) t;;

(** {6 Composing skeletons} *)

val farm : ('a, 'b) t * number_of_workers -> ('a, 'b) t;;
val farm_i : (worker_index -> ('a, 'b) t) * number_of_workers -> ('a, 'b) t;;
val farm_vector : ('a, 'b) t * number_of_workers -> ('a array, 'b array) t;;
val loop : ('a, bool) t * ('a, 'a) t -> ('a, 'a) t;;
val pipeline : ('a, 'b) t -> ('b, 'c) t -> ('a, 'c) t;;
val ( ||| ) : ('a, 'b) t -> ('b, 'c) t -> ('a, 'c) t;;
val product : ('a, 'b) t -> ('c, 'd) t -> ('a * 'c, 'b * 'd) t;;
val ( *** ) : ('a, 'b) t -> ('c, 'd) t -> ('a * 'c, 'b * 'd) t;;
val rails : ('a, 'b) t array -> ('a array, 'b array) t;;
val sum : ('a, 'c) t * ('b, 'c) t -> (('a, 'b) sum, 'c) t;;
val ( +++ ) : ('a, 'c) t -> ('b, 'c) t -> (('a, 'b) sum, 'c) t;;

(** {6 Coloring atom skeletons} *)

val colorize : ('a, 'b) t -> color -> ('a, 'b) t;;

(**/**)
(** {6 undocumented features} *)

val atom : 'x * ('x, 'a, 'b) Expor.t -> ('a, 'b) t;;
