(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: sklml_options.ml,v 1.16 2017/01/04 11:58:27 fclement Exp $ *)

module Nat = Unatural;;
module Perv = Upervasives;;
module Ar = Uaref;;
module Ok = Uoption_key;;
module Co = Ucommand_option;;

let _2_ = Nat._2_;;

let nat_uaref_make_aref mk_aref_int name init =
  let { Ar.get; set; } = mk_aref_int name (Nat.to_int init) in
  (* We will always have get () > 0.
     Therefore, the next Nat.unsafe_of_int will be safe. *)
  let get () = Nat.unsafe_of_int (get ()) in
  let set n = set (Nat.to_int n) in
  { Ar.get; set; }
;;

let qual = Perv.qualify "Options";;
let invalid_arg = Perv.invalid_arg_qual qual;;

let nat_set set n =
  if n < 0 then invalid_arg "nat_set" "negative number" else
  (* Now, we have n >= 0.
     Therefore, the next Nat.unsafe_of_int will be safe. *)
  set (Nat.unsafe_of_int n)
;;

let number_of_workers_option_name = "number of workers"
and default_number_of_workers = _2_
;;

let { Ar.get = get_number_of_workers; set = set_number_of_workers; } =
  nat_uaref_make_aref Ar.make_aref_single_positive_int
    number_of_workers_option_name
    default_number_of_workers
;;

let phase_sklml_number_of_workers =
  Co.Phase (Printf.sprintf "Sklml %s" number_of_workers_option_name)
;;

let { Ok.long = number_of_workers_lok; short = number_of_workers_sok_st; } =
  Ok.make_option_keys
    number_of_workers_option_name (Ok.Short (Ok.of_string "-nw"))
;;

let number_of_workers_rco =
  {
   Co.key = number_of_workers_lok;
   spec = Arg.Int (nat_set set_number_of_workers);
   doc =
     Printf.sprintf
       "<int>\n\
       \tSet the number of effective workers to <int>. Default is %d."
       (Nat.to_int (get_number_of_workers ()));
  }
;;

let add_option () =
  Co.application_add_all_rco_with_sk
    [
     (number_of_workers_sok_st, number_of_workers_rco);
    ];

  Co.phase_add phase_sklml_number_of_workers number_of_workers_lok;
;;

add_option ();;
