(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: skl_comm.mli,v 1.25 2017/01/04 11:58:29 fclement Exp $ *)

(** {3 Glue library between Sklml programs and the Sklml parallel launcher
 daemon} *)

open Tools_common_types;;
open Common_types;;

type request =
  | Atom_req of Obj.t * atom_id * color
  | Loop_req of Iotree.t * Iotree.t
  | Pipe_req of Iotree.t * Iotree.t
(** The type of requests. *)
;;

type reply =
  | Failure
  | Success of Unix.sockaddr
(** The type of replies. *)
;;

(** {6 Functions} *)

val request_daemon : IO.t -> request -> reply option;;
val extract_request : IO.ic -> request;;
val send_reply : IO.oc -> reply -> unit;;

val open_skl_fifo : bool -> pid -> IO.t;;
val clean_skl_fifo : pid -> unit;;
