(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: tools_common_types.mli,v 1.10 2017/05/16 18:45:31 fclement Exp $ *)

(** Types for the tools_common library. *)

type index = Upervasives.index
and length = Upervasives.length
and number_of_workers = length
and offset = Upervasives.offset
and pid = Unatural.t
and port = Unatural.t
and return_code = Unatural.t
and size = length
and timeout = Unatural.t
and worker_index = index
(** Aliases for type [Unatural.t]. *)
;;

(* Fix function placement in parallel/daemon/master.ml to be able to make
 type color an alias for Unatural.t. *)
type color = int
(** Alias for type [int]. *)
;;

type command = string
and command_kind = string
and command_suffix = suffix
and command_title = string
and doc_mandatory_arguments = string
and inc = path
and lib = path
and option_key = string
and path = string
and root_dir = path
and suffix = string
(** Aliases for type [string]. *)
;;

type semantics_mode =
  | Seq
  | Par
;;

type compilation_mode =
  | Byt
  | Bin
;;

type generator_kind =
  | Dep
  | Doc
;;

type message_print = Upervasives.message -> unit
and gen_message_print = Upervasives.gen_message -> unit
;;

type report = message_print
and report_action = gen_message_print
and debug = message_print
and debug_action = gen_message_print
and warning = message_print
and 'a error = Upervasives.message -> return_code -> 'a
;;

type 'a services =
  {
   report : report;
   debug : debug;
   error : 'a error;
  }
;;

type tagged_report = report
and tagged_report_action = report_action
and tagged_debug = Upervasives.function_name -> debug
and tagged_debug_action = Upervasives.function_name -> debug_action
and tagged_warning = warning
and 'a tagged_error = Upervasives.message -> 'a
;;

type 'a tagged_services =
  {
   tagged_report : tagged_report;
   tagged_report_action : tagged_report_action;
   tagged_debug : tagged_debug;
   tagged_debug_action : tagged_debug_action;
   tagged_warning : tagged_warning;
   tagged_error : 'a tagged_error;
  }
;;
