(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2007-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux                                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: tools_options.mli,v 1.14 2017/01/04 11:58:30 fclement Exp $ *)

open Tools_common_types;;

(** {6 Command-line options} *)

val compilation_mode_ok : Uoption_key.t;;
val semantics_mode_ok : Uoption_key.t;;
val sklml_install_root_dir_ok : Uoption_key.t;;
val sklml_lib_root_dir_ok : Uoption_key.t;;
val sklml_pph_tekno_dir_ok : Uoption_key.t;;

val compilation_mode_rco : Ucommand_option.regular_command_option;;
val semantics_mode_rco : Ucommand_option.regular_command_option;;
val sklml_install_root_dir_rco : Ucommand_option.regular_command_option;;
val sklml_lib_root_dir_rco : Ucommand_option.regular_command_option;;
val sklml_pph_tekno_dir_rco : Ucommand_option.regular_command_option;;

val get_compilation_mode : unit -> compilation_mode;;
val get_semantics_mode : unit -> semantics_mode;;
val get_sklml_lib_root_dir : unit -> string;;
val get_ocaml_options : Ucommand_option.command_line -> string list;;

val set_command_line_options :
  command_kind -> command_suffix ->
  Uoption_key.t list -> Ucommand_option.regular_command_option list ->
  doc_mandatory_arguments ->
  command_title * Ucommand_option.phase_name
;;

(** {6 Report/debug flags and warning/error messages} *)

val add_services :
  Ucommand_option.software_option_key_data -> Upervasives.software_name ->
  'a services
;;

val add_tagged_services : Upervasives.software_name -> 'a tagged_services;;

(** {6 Parsing the command line} *)

val parse :
  'a services -> Ucommand_option.phase_name ->
  (Ucommand_option.command_line -> 'a) -> 'a
;;
